@@ -25,11 +25,11 @@
  * SUCH DAMAGE.
  */
 
-#ifndef CONFIG_H
-#define CONFIG_H
+#ifndef DEFS_H
+#define DEFS_H
 
 #define PACKAGE			"dhcpcd"
-#define VERSION			"10.0.4"
+#define VERSION			"10.0.6"
 
 #ifndef PRIVSEP_USER
 # define PRIVSEP_USER		"_" PACKAGE
