@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Solaris interface driver for dhcpcd
- * Copyright (c) 2016-2020 Roy Marples <roy@marples.name>
+ * Copyright (c) 2016-2023 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -94,12 +94,6 @@ extern int getallifaddrs(sa_family_t, struct ifaddrs **, int64_t);
 
 #define COPYSA(dst, src) memcpy((dst), (src), sa_len((src)))
 
-struct priv {
-#ifdef INET6
-	int pf_inet6_fd;
-#endif
-};
-
 struct rtm
 {
 	struct rt_msghdr hdr;
@@ -533,6 +527,8 @@ if_findsa(struct dhcpcd_ctx *ctx, const struct sockaddr *sa)
 		sin = (const void *)sa;
 		if ((ia = ipv4_findmaskaddr(ctx, &sin->sin_addr)))
 			return ia->iface;
+		if ((ia = ipv4_findmaskbrd(ctx, &sin->sin_addr)))
+			return ia->iface;
 		break;
 	}
 #endif
