@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Privilege Separation for dhcpcd, privileged proxy
- * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2023 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -74,7 +74,7 @@ struct psr_ctx {
 };
 
 static void
-ps_root_readerrorcb(void *arg)
+ps_root_readerrorcb(void *arg, unsigned short events)
 {
 	struct psr_ctx *psr_ctx = arg;
 	struct dhcpcd_ctx *ctx = psr_ctx->psr_ctx;
@@ -87,6 +87,9 @@ ps_root_readerrorcb(void *arg)
 	ssize_t len;
 	int exit_code = EXIT_FAILURE;
 
+	if (events != ELE_READ)
+		logerrx("%s: unexpected event 0x%04x", __func__, events);
+
 #define PSR_ERROR(e)				\
 	do {					\
 		psr_error->psr_result = -1;	\
@@ -94,7 +97,7 @@ ps_root_readerrorcb(void *arg)
 		goto out;			\
 	} while (0 /* CONSTCOND */)
 
-	len = readv(ctx->ps_root_fd, iov, __arraycount(iov));
+	len = readv(PS_ROOT_FD(ctx), iov, __arraycount(iov));
 	if (len == -1)
 		PSR_ERROR(errno);
 	else if ((size_t)len < sizeof(*psr_error))
@@ -112,13 +115,15 @@ ps_root_readerror(struct dhcpcd_ctx *ctx, void *data, size_t len)
 	    .psr_ctx = ctx,
 	    .psr_data = data, .psr_datalen = len,
 	};
+	int fd = PS_ROOT_FD(ctx);
 
-	if (eloop_event_add(ctx->ps_eloop, ctx->ps_root_fd,
+	if (eloop_event_add(ctx->ps_eloop, fd, ELE_READ,
 	    ps_root_readerrorcb, &psr_ctx) == -1)
 		return -1;
 
 	eloop_enter(ctx->ps_eloop);
 	eloop_start(ctx->ps_eloop, &ctx->sigset);
+	eloop_event_delete(ctx->ps_eloop, fd);
 
 	errno = psr_ctx.psr_error.psr_errno;
 	return psr_ctx.psr_error.psr_result;
@@ -126,7 +131,7 @@ ps_root_readerror(struct dhcpcd_ctx *ctx, void *data, size_t len)
 
 #ifdef PRIVSEP_GETIFADDRS
 static void
-ps_root_mreaderrorcb(void *arg)
+ps_root_mreaderrorcb(void *arg, unsigned short events)
 {
 	struct psr_ctx *psr_ctx = arg;
 	struct dhcpcd_ctx *ctx = psr_ctx->psr_ctx;
@@ -138,7 +143,10 @@ ps_root_mreaderrorcb(void *arg)
 	ssize_t len;
 	int exit_code = EXIT_FAILURE;
 
-	len = recv(ctx->ps_root_fd, psr_error, sizeof(*psr_error), MSG_PEEK);
+	if (events != ELE_READ)
+		logerrx("%s: unexpected event 0x%04x", __func__, events);
+
+	len = recv(PS_ROOT_FD(ctx), psr_error, sizeof(*psr_error), MSG_PEEK);
 	if (len == -1)
 		PSR_ERROR(errno);
 	else if ((size_t)len < sizeof(*psr_error))
@@ -155,7 +163,7 @@ ps_root_mreaderrorcb(void *arg)
 		iov[1].iov_len = psr_ctx->psr_datalen;
 	}
 
-	len = readv(ctx->ps_root_fd, iov, __arraycount(iov));
+	len = readv(PS_ROOT_FD(ctx), iov, __arraycount(iov));
 	if (len == -1)
 		PSR_ERROR(errno);
 	else if ((size_t)len != sizeof(*psr_error) + psr_ctx->psr_datalen)
@@ -172,13 +180,15 @@ ps_root_mreaderror(struct dhcpcd_ctx *ctx, void **data, size_t *len)
 	struct psr_ctx psr_ctx = {
 	    .psr_ctx = ctx,
 	};
+	int fd = PS_ROOT_FD(ctx);
 
-	if (eloop_event_add(ctx->ps_eloop, ctx->ps_root_fd,
+	if (eloop_event_add(ctx->ps_eloop, fd, ELE_READ,
 	    ps_root_mreaderrorcb, &psr_ctx) == -1)
 		return -1;
 
 	eloop_enter(ctx->ps_eloop);
 	eloop_start(ctx->ps_eloop, &ctx->sigset);
+	eloop_event_delete(ctx->ps_eloop, fd);
 
 	errno = psr_ctx.psr_error.psr_errno;
 	*data = psr_ctx.psr_data;
@@ -201,12 +211,13 @@ ps_root_writeerror(struct dhcpcd_ctx *ctx, ssize_t result,
 		{ .iov_base = data, .iov_len = len },
 	};
 	ssize_t err;
+	int fd = PS_ROOT_FD(ctx);
 
 #ifdef PRIVSEP_DEBUG
 	logdebugx("%s: result %zd errno %d", __func__, result, errno);
 #endif
 
-	err = writev(ctx->ps_root_fd, iov, __arraycount(iov));
+	err = writev(fd, iov, __arraycount(iov));
 
 	/* Error sending the message? Try sending the error of sending. */
 	if (err == -1) {
@@ -216,7 +227,7 @@ ps_root_writeerror(struct dhcpcd_ctx *ctx, ssize_t result,
 		psr.psr_errno = errno;
 		iov[1].iov_base = NULL;
 		iov[1].iov_len = 0;
-		err = writev(ctx->ps_root_fd, iov, __arraycount(iov));
+		err = writev(fd, iov, __arraycount(iov));
 	}
 
 	return err;
@@ -284,6 +295,7 @@ ps_root_run_script(struct dhcpcd_ctx *ctx, const void *data, size_t len)
 	pid = script_exec(argv, ctx->script_env);
 	if (pid == -1)
 		return -1;
+
 	/* Wait for the script to finish */
 	while (waitpid(pid, &status, 0) == -1) {
 		if (errno != EINTR) {
@@ -483,12 +495,13 @@ ps_root_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 
 	if (psp != NULL) {
 		if (psm->ps_cmd & PS_STOP) {
-			int ret = ps_dostop(ctx, &psp->psp_pid, &psp->psp_fd);
-
-			ps_freeprocess(psp);
-			return ret;
+			return ps_stopprocess(psp);
 		} else if (psm->ps_cmd & PS_START) {
 			/* Process has already started .... */
+			logdebugx("%s%sprocess %s already started on pid %d",
+			    psp->psp_ifname,
+			    psp->psp_ifname[0] != '\0' ? ": " : "",
+			    psp->psp_name, psp->psp_pid);
 			return 0;
 		}
 
@@ -496,9 +509,6 @@ ps_root_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 		if (err == -1) {
 			logerr("%s: failed to send message to pid %d",
 			    __func__, psp->psp_pid);
-			shutdown(psp->psp_fd, SHUT_RDWR);
-			close(psp->psp_fd);
-			psp->psp_fd = -1;
 			ps_freeprocess(psp);
 		}
 		return 0;
@@ -546,6 +556,14 @@ ps_root_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 	case PS_SCRIPT:
 		err = ps_root_run_script(ctx, data, len);
 		break;
+	case PS_STOPPROCS:
+		ctx->options |= DHCPCD_EXITING;
+		TAILQ_FOREACH(psp, &ctx->ps_processes, next) {
+			if (psp != ctx->ps_root)
+				ps_stopprocess(psp);
+		}
+		err = ps_stopwait(ctx);
+		break;
 	case PS_UNLINK:
 		if (!ps_root_validpath(ctx, psm->ps_cmd, data)) {
 			err = -1;
@@ -607,7 +625,7 @@ ps_root_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 		break;
 #endif
 	default:
-		err = ps_root_os(psm, msg, &rdata, &rlen, &free_rdata);
+		err = ps_root_os(ctx, psm, msg, &rdata, &rlen, &free_rdata);
 		break;
 	}
 
@@ -619,11 +637,12 @@ ps_root_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 
 /* Receive from state engine, do an action. */
 static void
-ps_root_recvmsg(void *arg)
+ps_root_recvmsg(void *arg, unsigned short events)
 {
-	struct dhcpcd_ctx *ctx = arg;
+	struct ps_process *psp = arg;
 
-	if (ps_recvpsmsg(ctx, ctx->ps_root_fd, ps_root_recvmsgcb, ctx) == -1)
+	if (ps_recvpsmsg(psp->psp_ctx, psp->psp_fd, events,
+	    ps_root_recvmsgcb, psp->psp_ctx) == -1)
 		logerr(__func__);
 }
 
@@ -645,15 +664,15 @@ ps_root_handleinterface(void *arg, int action, const char *ifname)
 		return -1;
 	}
 
-	return (int)ps_sendcmd(ctx, ctx->ps_data_fd, PS_DEV_IFCMD, flag,
-	    ifname, strlen(ifname) + 1);
+	return (int)ps_sendcmd(ctx, ctx->ps_data_fd, PS_DEV_IFCMD,
+	    flag, ifname, strlen(ifname) + 1);
 }
 #endif
 
 static int
-ps_root_startcb(void *arg)
+ps_root_startcb(struct ps_process *psp)
 {
-	struct dhcpcd_ctx *ctx = arg;
+	struct dhcpcd_ctx *ctx = psp->psp_ctx;
 
 	if (ctx->options & DHCPCD_MANAGER)
 		setproctitle("[privileged proxy]");
@@ -662,15 +681,28 @@ ps_root_startcb(void *arg)
 		    ctx->ifv[0],
 		    ctx->options & DHCPCD_IPV4 ? " [ip4]" : "",
 		    ctx->options & DHCPCD_IPV6 ? " [ip6]" : "");
-	ctx->ps_root_pid = getpid();
 	ctx->options |= DHCPCD_PRIVSEPROOT;
 
+	if (if_opensockets(ctx) == -1)
+		logerr("%s: if_opensockets", __func__);
+#ifdef BSD
+	else {
+		/* We only want to write to this socket, so set
+		 * a small as possible buffer size. */
+		socklen_t smallbuf = 1;
+
+		if (setsockopt(ctx->link_fd, SOL_SOCKET, SO_RCVBUF,
+		    &smallbuf, (socklen_t)sizeof(smallbuf)) == -1)
+			logerr("%s: setsockopt(SO_RCVBUF)", __func__);
+	}
+#endif
+
 	/* Open network sockets for sending.
 	 * This is a small bit wasteful for non sandboxed OS's
 	 * but makes life very easy for unicasting DHCPv6 in non manager
 	 * mode as we no longer care about address selection.
 	 * We can't call shutdown SHUT_RD on the socket because it's
-	 * not connectd. All we can do is try and set a zero sized
+	 * not connected. All we can do is try and set a zero sized
 	 * receive buffer and just let it overflow.
 	 * Reading from it just to drain it is a waste of CPU time. */
 #ifdef INET
@@ -722,15 +754,54 @@ ps_root_startcb(void *arg)
 	return 0;
 }
 
-static void
-ps_root_signalcb(int sig, __unused void *arg)
+void
+ps_root_signalcb(int sig, void *arg)
 {
+	struct dhcpcd_ctx *ctx = arg;
+	int status;
+	pid_t pid;
+	const char *ifname, *name;
+	struct ps_process *psp;
 
-	if (sig == SIGCHLD) {
-		while (waitpid(-1, NULL, WNOHANG) > 0)
-			;
+	if (sig != SIGCHLD)
 		return;
+
+	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
+		psp = ps_findprocesspid(ctx, pid);
+		if (psp != NULL) {
+			ifname = psp->psp_ifname;
+			name = psp->psp_name;
+		} else {
+			/* Ignore logging the double fork */
+			if (ctx->options & DHCPCD_LAUNCHER)
+				continue;
+			ifname = "";
+			name = "unknown process";
+		}
+
+		if (WIFEXITED(status) && WEXITSTATUS(status) != 0)
+			logerrx("%s%s%s exited unexpectedly from PID %d,"
+			    " code=%d",
+			    ifname, ifname[0] != '\0' ? ": " : "",
+			    name, pid, WEXITSTATUS(status));
+		else if (WIFSIGNALED(status))
+			logerrx("%s%s%s exited unexpectedly from PID %d,"
+			    " signal=%s",
+			    ifname, ifname[0] != '\0' ? ": " : "",
+			    name, pid, strsignal(WTERMSIG(status)));
+		else
+			logdebugx("%s%s%s exited from PID %d",
+			    ifname, ifname[0] != '\0' ? ": " : "",
+			    name, pid);
+
+		if (psp != NULL)
+			ps_freeprocess(psp);
 	}
+
+	if (!(ctx->options & DHCPCD_EXITING))
+		return;
+	if (!(ps_waitforprocs(ctx)))
+		eloop_exit(ctx->ps_eloop, EXIT_SUCCESS);
 }
 
 int (*handle_interface)(void *, int, const char *);
@@ -789,37 +860,46 @@ ps_root_dispatchcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 }
 
 static void
-ps_root_dispatch(void *arg)
+ps_root_dispatch(void *arg, unsigned short events)
 {
 	struct dhcpcd_ctx *ctx = arg;
 
-	if (ps_recvpsmsg(ctx, ctx->ps_data_fd, ps_root_dispatchcb, ctx) == -1)
+	if (ps_recvpsmsg(ctx, ctx->ps_data_fd, events,
+	    ps_root_dispatchcb, ctx) == -1)
 		logerr(__func__);
 }
 
 static void
-ps_root_log(void *arg)
+ps_root_log(void *arg, unsigned short events)
 {
 	struct dhcpcd_ctx *ctx = arg;
 
-	if (logreadfd(ctx->ps_log_fd) == -1)
+	if (events != ELE_READ)
+		logerrx("%s: unexpected event 0x%04x", __func__, events);
+
+	if (logreadfd(ctx->ps_log_root_fd) == -1)
 		logerr(__func__);
 }
 
 pid_t
 ps_root_start(struct dhcpcd_ctx *ctx)
 {
+	struct ps_id id = {
+		.psi_ifindex = 0,
+		.psi_cmd = PS_ROOT,
+	};
+	struct ps_process *psp;
 	int logfd[2], datafd[2];
 	pid_t pid;
 
-	if (xsocketpair(AF_UNIX, SOCK_DGRAM | SOCK_CXNB, 0, logfd) == -1)
+	if (xsocketpair(AF_UNIX, SOCK_SEQPACKET | SOCK_CXNB, 0, logfd) == -1)
 		return -1;
 #ifdef PRIVSEP_RIGHTS
 	if (ps_rights_limit_fdpair(logfd) == -1)
 		return -1;
 #endif
 
-	if (socketpair(AF_UNIX, SOCK_DGRAM | SOCK_CXNB, 0, datafd) == -1)
+	if (socketpair(AF_UNIX, SOCK_SEQPACKET | SOCK_CXNB, 0, datafd) == -1)
 		return -1;
 	if (ps_setbuf_fdpair(datafd) == -1)
 		return -1;
@@ -828,16 +908,17 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 		return -1;
 #endif
 
-	pid = ps_dostart(ctx, &ctx->ps_root_pid, &ctx->ps_root_fd,
-	    ps_root_recvmsg, NULL, ctx,
-	    ps_root_startcb, ps_root_signalcb, 0);
+	psp = ctx->ps_root = ps_newprocess(ctx, &id);
+	strlcpy(psp->psp_name, "privileged proxy", sizeof(psp->psp_name));
+	pid = ps_startprocess(psp, ps_root_recvmsg, NULL,
+	    ps_root_startcb, ps_root_signalcb, PSF_ELOOP);
 
 	if (pid == 0) {
-		ctx->ps_log_fd = logfd[1];
-		if (eloop_event_add(ctx->eloop, ctx->ps_log_fd,
+		ctx->ps_log_fd = logfd[0]; /* Keep open to pass to processes */
+		ctx->ps_log_root_fd = logfd[1];
+		if (eloop_event_add(ctx->eloop, ctx->ps_log_root_fd, ELE_READ,
 		    ps_root_log, ctx) == -1)
 			return -1;
-		close(logfd[0]);
 		ctx->ps_data_fd = datafd[1];
 		close(datafd[0]);
 		return 0;
@@ -849,16 +930,9 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 
 	ctx->ps_data_fd = datafd[0];
 	close(datafd[1]);
-	if (eloop_event_add(ctx->eloop, ctx->ps_data_fd,
+	if (eloop_event_add(ctx->eloop, ctx->ps_data_fd, ELE_READ,
 	    ps_root_dispatch, ctx) == -1)
-		return -1;
-
-	if ((ctx->ps_eloop = eloop_new()) == NULL)
-		return -1;
-
-	eloop_signal_set_cb(ctx->ps_eloop,
-	    dhcpcd_signals, dhcpcd_signals_len,
-	    ps_root_signalcb, ctx);
+		return 1;
 
 	return pid;
 }
@@ -866,15 +940,60 @@ ps_root_start(struct dhcpcd_ctx *ctx)
 int
 ps_root_stop(struct dhcpcd_ctx *ctx)
 {
+	struct ps_process *psp = ctx->ps_root;
 
-	return ps_dostop(ctx, &ctx->ps_root_pid, &ctx->ps_root_fd);
+	if (!(ctx->options & DHCPCD_PRIVSEP) ||
+	    ctx->eloop == NULL)
+		return 0;
+
+	/* If we are the root process then remove the pidfile */
+	if (ctx->options & DHCPCD_PRIVSEPROOT &&
+	    !(ctx->options & DHCPCD_TEST))
+	{
+		if (unlink(ctx->pidfile) == -1)
+			logerr("%s: unlink: %s", __func__, ctx->pidfile);
+	}
+
+	/* Only the manager process gets past this point. */
+	if (ctx->options & DHCPCD_FORKED)
+		return 0;
+
+	/* We cannot log the root process exited before we
+	 * log dhcpcd exits because the latter requires the former.
+	 * So we just log the intent to exit.
+	 * Even sending this will be a race to exit. */
+	if (psp) {
+		logdebugx("%s%s%s will exit from PID %d",
+		    psp->psp_ifname,
+		    psp->psp_ifname[0] != '\0' ? ": " : "",
+		    psp->psp_name, psp->psp_pid);
+
+		if (ps_stopprocess(psp) == -1)
+			return -1;
+	} /* else the root process has already exited :( */
+
+	return ps_stopwait(ctx);
+}
+
+ssize_t
+ps_root_stopprocesses(struct dhcpcd_ctx *ctx)
+{
+
+	if (!(IN_PRIVSEP_SE(ctx)))
+		return 0;
+
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_STOPPROCS, 0,
+	    NULL, 0) == -1)
+		return -1;
+	return ps_root_readerror(ctx, NULL, 0);
 }
 
 ssize_t
 ps_root_script(struct dhcpcd_ctx *ctx, const void *data, size_t len)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_SCRIPT, 0, data, len) == -1)
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_SCRIPT,
+	    0, data, len) == -1)
 		return -1;
 	return ps_root_readerror(ctx, NULL, 0);
 }
@@ -883,14 +1002,15 @@ ssize_t
 ps_root_ioctl(struct dhcpcd_ctx *ctx, ioctl_request_t req, void *data,
     size_t len)
 {
+	int fd = PS_ROOT_FD(ctx);
 #ifdef IOCTL_REQUEST_TYPE
 	unsigned long ulreq = 0;
 
 	memcpy(&ulreq, &req, sizeof(req));
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_IOCTL, ulreq, data, len) == -1)
+	if (ps_sendcmd(ctx, fd, PS_IOCTL, ulreq, data, len) == -1)
 		return -1;
 #else
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_IOCTL, req, data, len) == -1)
+	if (ps_sendcmd(ctx, fd, PS_IOCTL, req, data, len) == -1)
 		return -1;
 #endif
 	return ps_root_readerror(ctx, data, len);
@@ -900,7 +1020,7 @@ ssize_t
 ps_root_unlink(struct dhcpcd_ctx *ctx, const char *file)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_UNLINK, 0,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_UNLINK, 0,
 	    file, strlen(file) + 1) == -1)
 		return -1;
 	return ps_root_readerror(ctx, NULL, 0);
@@ -910,7 +1030,7 @@ ssize_t
 ps_root_readfile(struct dhcpcd_ctx *ctx, const char *file,
     void *data, size_t len)
 {
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_READFILE, 0,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_READFILE, 0,
 	    file, strlen(file) + 1) == -1)
 		return -1;
 	return ps_root_readerror(ctx, data, len);
@@ -931,7 +1051,7 @@ ps_root_writefile(struct dhcpcd_ctx *ctx, const char *file, mode_t mode,
 	}
 	memcpy(buf + flen, data, len);
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_WRITEFILE, mode,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_WRITEFILE, mode,
 	    buf, flen + len) == -1)
 		return -1;
 	return ps_root_readerror(ctx, NULL, 0);
@@ -941,7 +1061,7 @@ ssize_t
 ps_root_filemtime(struct dhcpcd_ctx *ctx, const char *file, time_t *time)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_FILEMTIME, 0,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_FILEMTIME, 0,
 	    file, strlen(file) + 1) == -1)
 		return -1;
 	return ps_root_readerror(ctx, time, sizeof(*time));
@@ -951,7 +1071,8 @@ ssize_t
 ps_root_logreopen(struct dhcpcd_ctx *ctx)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_LOGREOPEN, 0, NULL, 0) == -1)
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_LOGREOPEN, 0,
+	    NULL, 0) == -1)
 		return -1;
 	return ps_root_readerror(ctx, NULL, 0);
 }
@@ -967,7 +1088,7 @@ ps_root_getifaddrs(struct dhcpcd_ctx *ctx, struct ifaddrs **ifahead)
 	size_t len;
 	ssize_t err;
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx),
 	    PS_GETIFADDRS, 0, NULL, 0) == -1)
 		return -1;
 	err = ps_root_mreaderror(ctx, &buf, &len);
@@ -1042,7 +1163,7 @@ ssize_t
 ps_root_ip6forwarding(struct dhcpcd_ctx *ctx, const char *ifname)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_IP6FORWARDING, 0,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_IP6FORWARDING, 0,
 	    ifname, ifname != NULL ? strlen(ifname) + 1 : 0) == -1)
 		return -1;
 	return ps_root_readerror(ctx, NULL, 0);
@@ -1054,7 +1175,7 @@ int
 ps_root_getauthrdm(struct dhcpcd_ctx *ctx, uint64_t *rdm)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_AUTH_MONORDM, 0,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_AUTH_MONORDM, 0,
 	    rdm, sizeof(*rdm))== -1)
 		return -1;
 	return (int)ps_root_readerror(ctx, rdm, sizeof(*rdm));
@@ -1066,7 +1187,7 @@ int
 ps_root_dev_initialised(struct dhcpcd_ctx *ctx, const char *ifname)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_DEV_INITTED, 0,
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_DEV_INITTED, 0,
 	    ifname, strlen(ifname) + 1)== -1)
 		return -1;
 	return (int)ps_root_readerror(ctx, NULL, 0);
@@ -1076,7 +1197,8 @@ int
 ps_root_dev_listening(struct dhcpcd_ctx * ctx)
 {
 
-	if (ps_sendcmd(ctx, ctx->ps_root_fd, PS_DEV_LISTENING, 0, NULL, 0)== -1)
+	if (ps_sendcmd(ctx, PS_ROOT_FD(ctx), PS_DEV_LISTENING,
+	    0, NULL, 0) == -1)
 		return -1;
 	return (int)ps_root_readerror(ctx, NULL, 0);
 }
